@@ -2,11 +2,11 @@
 
 import collections
 import concurrent.futures as cf
-import contextlib
 import dataclasses
 import datetime
 import functools
 import itertools
+import json
 import os
 import re
 import shutil
@@ -14,17 +14,18 @@ import tempfile
 import uuid
 import warnings
 import zipfile
-from typing import Dict, Iterable, List, Optional, Set, Tuple, Union
+from typing import Dict, Generator, Iterable, List, Optional, Set, Tuple, Union
 
 import requests
 from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 from dateutil.parser import parse as parse_date
 from dateutil.parser import ParserError
+from tabulate import tabulate
 
 import pylangacq
-from pylangacq.measures import _CLITIC, _get_ipsyn, _get_mlum, _get_mluw, _get_ttr
-from pylangacq.objects import Gra, Token, Utterance
+from pylangacq.measures import _get_ipsyn, _get_mlum, _get_mluw, _get_ttr
+from pylangacq.objects import Gra, Token, Utterance, _sort_keys, _CLITIC
 from pylangacq._clean_utterance import _clean_utterance
 
 
@@ -34,6 +35,13 @@ _CHAT_EXTENSION = ".cha"
 
 _TIMER_MARKS_REGEX = re.compile(r"\x15-?(\d+)_(\d+)-?\x15")
 
+_CACHED_DATA_DIR = os.path.join(os.path.expanduser("~"), ".pylangacq")
+_CACHED_DATA_JSON_PATH = os.path.join(_CACHED_DATA_DIR, "cached_data.json")
+
+_CHAT_LINE_INDICATORS = frozenset({"@", "*", "%"})
+
+_HEADER_REGEX = re.compile(r"\A@([^@:]+)(:\s+(\S[\S\s]+))?\Z")
+
 
 def _params_in_docstring(*params, class_method=True):
     docstring = ""
@@ -101,12 +109,6 @@ def _params_in_docstring(*params, class_method=True):
             If provided, the file paths that match this string (by regular expression
             matching) are excluded for reading and parsing."""
 
-    if "allow_remote" in params:
-        docstring += """
-        allow_remote : bool, optional
-            If ``True`` (the default), and if the data source looks like a URL,
-            downloading the data from the internet will be attempted."""
-
     if "encoding" in params:
         docstring += """
         encoding : str, optional
@@ -138,15 +140,47 @@ def _params_in_docstring(*params, class_method=True):
             and further parallelization from within PyLangAcq might be undesirable),
             you may like to consider setting this parameter to ``False``."""
 
+    if "use_cached" in params:
+        docstring += """
+        use_cached : bool, optional
+            If ``True`` (the default), and if the path is a URL for a remote ZIP
+            archive, then CHAT reading attempts to use the previously downloaded
+            data cached on disk. This setting allows you to call
+            this function with the same URL repeatedly without hitting the CHILDES /
+            TalkBank server more than once for the same data.
+            Pass in ``False`` to force a new download; the upstream CHILDES / TalkBank
+            data is updated in minor ways from time to time, e.g., for CHAT format,
+            header/metadata information, updated annotations.
+            See also the helper functions: :func:`pylangacq.chat.cached_data_info`,
+            :func:`pylangacq.chat.remove_cached_data`."""
+
+    if "session" in params:
+        docstring += """
+        session : requests.Session, optional
+            If the path is a URL for a remote ZIP archive, data downloading is
+            done with reasonable settings of retries and timeout by default,
+            in order to be robust against intermittent network issues.
+            If necessary, pass in your own instance of :class:`requests.Session`
+            to customize."""
+
     if not class_method:
         docstring = docstring.replace("\n        ", "\n    ")
 
     def real_decorator(func):
-        if class_method:
-            returns_header = "\n\n        Returns\n        -------"
+        returns_none = (
+            f"Returns\n{'        ' if class_method else '    '}-------"
+            not in func.__doc__
+        )
+        if returns_none:
+            func.__doc__ += docstring
         else:
-            returns_header = "\n\n    Returns\n    -------"
-        func.__doc__ = func.__doc__.replace(returns_header, docstring + returns_header)
+            if class_method:
+                returns_header = "\n\n        Returns\n        -------"
+            else:
+                returns_header = "\n\n    Returns\n    -------"
+            func.__doc__ = func.__doc__.replace(
+                returns_header, docstring + returns_header
+            )
 
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
@@ -179,6 +213,14 @@ def _deprecate_warning(what, since_version, use_instead):
     )
 
 
+class _list(list):
+    def __repr__(self):
+        return "\n".join(x._to_str() for x in self)
+
+    def _repr_html_(self) -> str:
+        return "\n".join(x._repr_html_() for x in self)
+
+
 @dataclasses.dataclass
 class _File:
     """A CHAT file (or string).
@@ -224,7 +266,7 @@ class Reader:
             "Number of files in this reader? Utterances? Words? Something else?"
         )
 
-    def _get_reader_from_files(self, files: Iterable[_File]):
+    def _get_reader_from_files(self, files: Iterable[_File]) -> "pylangacq.Reader":
         reader = self.__class__()
         reader._files = collections.deque(files)
         return reader
@@ -258,6 +300,11 @@ class Reader:
         """Remove all data from this reader."""
         self._files = collections.deque()
 
+    def __add__(self, other: "pylangacq.Reader") -> "pylangacq.Reader":
+        if not issubclass(other.__class__, Reader):
+            raise TypeError(f'cannot concatenate "{other.__class__}" to a reader')
+        return self._get_reader_from_files(self._files + other._files)
+
     def _append(self, left_or_right, reader: "pylangacq.Reader") -> None:
         func = "extendleft" if left_or_right == "left" else "extend"
         if not issubclass(reader.__class__, Reader):
@@ -586,7 +633,7 @@ class Reader:
         Set[datetime.date] if ``by_files`` is ``False``,
         otherwise List[Set[datetime.date]]]
         """
-        result_by_files = [f.header["Date"] for f in self._files]
+        result_by_files = [f.header.get("Date", set()) for f in self._files]
         if by_files:
             return result_by_files
         else:
@@ -912,7 +959,7 @@ class Reader:
         """
         strs = list(strs)
         if ids is None:
-            ids = [str(uuid.uuid4()) for _ in range(len(strs))]
+            ids = [_get_uuid() for _ in range(len(strs))]
         else:
             ids = list(ids)
         if len(strs) != len(ids):
@@ -1017,7 +1064,13 @@ class Reader:
 
     @classmethod
     @_params_in_docstring(
-        "match", "exclude", "extension", "allow_remote", "encoding", "parallel"
+        "match",
+        "exclude",
+        "extension",
+        "encoding",
+        "parallel",
+        "use_cached",
+        "session",
     )
     def from_zip(
         cls,
@@ -1025,12 +1078,19 @@ class Reader:
         match: str = None,
         exclude: str = None,
         extension: str = _CHAT_EXTENSION,
-        allow_remote: bool = True,
         encoding: str = _ENCODING,
         parallel: bool = True,
+        use_cached: bool = True,
+        session: requests.Session = None,
     ) -> "pylangacq.Reader":
         """Instantiate a reader from a local or remote ZIP file.
 
+        If the input data is a remote ZIP file and you expect to call this method
+        with the same path multiple times, consider downloading the data to the local
+        system and then reading it from there to avoid unnecessary re-downloading.
+        Caching a remote ZIP file isn't implemented (yet) as the upstream CHILDES /
+        TalkBank data is updated in minor ways from time to time.
+
         Parameters
         ----------
         path : str
@@ -1044,24 +1104,28 @@ class Reader:
         -------
         :class:`pylangacq.Reader`
         """
-        with contextlib.ExitStack() as stack:
-            temp_dir = stack.enter_context(tempfile.TemporaryDirectory())
+        with tempfile.TemporaryDirectory() as temp_dir:
             is_url = path.startswith("https://") or path.startswith("http://")
+            unzip_dir = cls._retrieve_unzip_dir(path) if is_url else None
 
-            if allow_remote and is_url:
+            if is_url and (not use_cached or not unzip_dir):
+                if unzip_dir:
+                    remove_cached_data(path)
                 zip_path = os.path.join(temp_dir, os.path.basename(path))
-                _download_file(path, zip_path)
+                _download_file(path, zip_path, session)
+                unzip_dir = cls._create_unzip_dir(path)
+            elif is_url and unzip_dir:
+                zip_path = None
             else:
                 zip_path = path
+                unzip_dir = temp_dir
 
-            zfile = stack.enter_context(zipfile.ZipFile(zip_path))
-            zfile.extractall(temp_dir)
-
-            if allow_remote and is_url:
-                os.remove(zip_path)
+            if zip_path:
+                with zipfile.ZipFile(zip_path) as zfile:
+                    zfile.extractall(unzip_dir)
 
             reader = cls.from_dir(
-                temp_dir,
+                unzip_dir,
                 match=match,
                 exclude=exclude,
                 extension=extension,
@@ -1071,10 +1135,294 @@ class Reader:
 
         # Unzipped files from `.from_zip` have the unwieldy temp dir in the file path.
         for f in reader._files:
-            f.file_path = f.file_path.replace(temp_dir, "").lstrip(os.sep)
+            f.file_path = f.file_path.replace(unzip_dir, "").lstrip(os.sep)
 
         return reader
 
+    @staticmethod
+    def _retrieve_unzip_dir(url: str) -> Union[str, None]:
+        try:
+            existing_records = json.load(open(_CACHED_DATA_JSON_PATH, encoding="utf-8"))
+        except FileNotFoundError:
+            return None
+        subdir = existing_records.get(url, {}).get("subdir")
+        if subdir is None:
+            return None
+        else:
+            return os.path.join(_CACHED_DATA_DIR, subdir)
+
+    @staticmethod
+    def _create_unzip_dir(url: str) -> str:
+        if not os.path.isdir(_CACHED_DATA_DIR):
+            _initialize_cached_data_dir()
+
+        subdir = _get_uuid()
+        unzip_dir = os.path.join(_CACHED_DATA_DIR, subdir)
+        os.makedirs(unzip_dir)
+
+        new_record = {
+            "subdir": subdir,
+            "url": url,
+            "cached_at": datetime.datetime.now().isoformat(),
+        }
+        existing_records = json.load(open(_CACHED_DATA_JSON_PATH, encoding="utf-8"))
+        _write_cached_data_json({**existing_records, **{url: new_record}})
+
+        return unzip_dir
+
+    def to_strs(self, tabular: bool = True) -> Generator[str, None, None]:
+        """Yield CHAT data strings.
+
+        .. note::
+            The header information may not be completely reproduced in the output
+            CHAT strings. Known issues all have to do with a header field used
+            multiple times in the original CHAT data.
+            For ``Date``, only the first date of recording is retained in the output
+            string.
+            For all other multiply used header fields
+            (e.g., ``Tape Location``, ``Time Duration``),
+            only the last value in a given CHAT file is retained.
+            Note that ``ID`` for participant information is not affected.
+
+        Parameters
+        ----------
+        tabular : bool, optional
+            If ``True``, adjust spacing such that the three tiers of the utterance,
+            %mor, and %gra are aligned in a tabular form. Note that such alignment
+            would drop annotations (e.g., pauses) on the main utterance tier.
+
+        Yields
+        ------
+        str
+            CHAT data string for one file.
+        """
+        header_first = (
+            "UTF8",
+            "PID",
+            "Languages",
+            "Participants",
+            "Date",
+            "Types",
+        )
+
+        for f in self._files:
+            str_for_file = ""
+
+            header_keys = _sort_keys(f.header.keys(), first=header_first)
+
+            for key in header_keys:
+                if key == "Languages":
+                    str_for_file += (
+                        f"@Languages:\t{' , '.join(f.header['Languages'])}\n"
+                    )
+                elif key == "Participants":
+                    participants = f.header["Participants"]
+
+                    parts = []
+                    for code, demographics in participants.items():
+                        parts.append(
+                            f"{code} {demographics['name']} {demographics['role']}"
+                        )
+                    str_for_file += f"@Participants:\t{' , '.join(parts)}\n"
+
+                    for code, d in participants.items():
+                        # d = demographics
+                        id_line = (
+                            f"{d['language']}|"
+                            f"{d['corpus']}|"
+                            f"{code}|"
+                            f"{d['age']}|"
+                            f"{d['sex']}|"
+                            f"{d['group']}|"
+                            f"{d['ses']}|"
+                            f"{d['role']}|"
+                            f"{d['education']}|"
+                            f"{d['custom']}|"
+                        )
+                        str_for_file += f"@ID:\t{id_line}\n"
+
+                elif key == "Date":
+                    # TODO: A CHAT file may have more than one recording date.
+                    try:
+                        date = sorted(f.header["Date"])[0]
+                    except IndexError:
+                        continue
+                    str_for_file += f"@Date:\t{date.strftime('%d-%b-%Y').upper()}\n"
+
+                elif f.header[key]:
+                    str_for_file += f"@{key}:\t{f.header[key]}\n"
+
+                else:
+                    str_for_file += f"@{key}\n"
+
+            for u in f.utterances:
+                str_for_file += u._to_str(tabular=tabular)
+
+            yield str_for_file
+
+    @_params_in_docstring("participants", "exclude")
+    def head(self, n: int = 5, participants=None, exclude=None):
+        """Return the first several utterances.
+
+        Parameters
+        ----------
+        n : int, optional
+            The number of utterances to return.
+
+        Returns
+        -------
+        list of utterances
+        """
+        return self._head_or_tail(slice(n), participants, exclude)
+
+    @_params_in_docstring("participants", "exclude")
+    def tail(self, n: int = 5, participants=None, exclude=None):
+        """Return the last several utterances.
+
+        Parameters
+        ----------
+        n : int, optional
+            The number of utterances to return.
+
+        Returns
+        -------
+        list of utterances
+        """
+        return self._head_or_tail(slice(-n, None), participants, exclude)
+
+    def _head_or_tail(self, slice_, participants, exclude):
+        us = self.utterances(
+            participants=participants, exclude=exclude, by_files=False
+        ).__getitem__(slice_)
+        return _list(us)
+
+    def _get_info_summary(self) -> str:
+        lines = [
+            f"{len(self._files)} files",
+            f"{sum(len(f.utterances) for f in self._files)} utterances",
+            f"{sum(len(u.tokens) for f in self._files for u in f.utterances)} words",
+        ]
+        return "\n".join(lines)
+
+    def _get_info_details_of_file(self, f: _File) -> Dict:
+        result = {
+            "Utterance Count": len(f.utterances),
+            "Word Count": sum(len(u.tokens) for u in f.utterances),
+        }
+        if not _is_uuid(f.file_path):
+            result["File Path"] = f.file_path
+        return result
+
+    def info(self, verbose=False) -> None:
+        """Print a summary of this Reader's data.
+
+        Parameters
+        ----------
+        verbose : bool, optional
+            If ``True`` (default is ``False``), show the details of all the files.
+        """
+        print(self._get_info_summary())
+        if len(self._files) < 2:
+            return
+        details = [self._get_info_details_of_file(f) for f in self._files]
+        max_n_files_if_not_verbose = 5
+        if not verbose:
+            details = details[:max_n_files_if_not_verbose]
+            n_files = max_n_files_if_not_verbose
+        else:
+            n_files = self.n_files()
+        indices = (f"#{i + 1}" for i in range(n_files))
+        output = tabulate(details, headers="keys", showindex=indices)
+        if not verbose:
+            output += "\n...\n(set `verbose` to True for all the files)"
+        print(output)
+
+    def to_chat(
+        self,
+        path: str,
+        is_dir: bool = False,
+        filenames: Iterable[str] = None,
+        tabular: bool = True,
+        encoding: str = _ENCODING,
+    ) -> None:
+        """Export to CHAT data files.
+
+        Parameters
+        ----------
+        path : str
+            The path to a file where you want to output the CHAT data,
+            e.g., `"data.cha"`, `"foo/bar/data.cha"`.
+        is_dir : bool, optional
+            If ``True`` (default is ``False``), then ``path`` is interpreted as a
+            directory instead.
+            The CHAT data is written to possibly multiple files under this directory.
+            The number of files you get can be checked by calling
+            :func:`~pylangacq.Reader.n_files`, which depends on how this reader
+            object is created.
+        filenames : Iterable[str], optional
+            Used only when ``is_dir`` is ``True``.
+            These are the filenames of the CHAT files to write.
+            If ``None`` or not given, {0001.cha, 0002.cha, ...} are used.
+        tabular : bool, optional
+            If ``True``, adjust spacing such that the three tiers of the utterance,
+            %mor, and %gra are aligned in a tabular form. Note that such alignment
+            would drop annotations (e.g., pauses) on the main utterance tier.
+        encoding : str, optional
+            Text encoding to output the CHAT data as. The default value is ``"utf-8"``
+            for Unicode UTF-8.
+
+        Raises
+        ------
+        ValueError
+            - If you attempt to output data to a single local file, but the CHAT
+              data in this reader appears to be organized in multiple files.
+            - If you attempt to output data to a directory while providing your own
+              filenames, but the number of your filenames doesn't match
+              the number of CHAT files in this reader object.
+        """
+        if not is_dir:
+            if self.n_files() > 1:
+                raise ValueError(
+                    "The CHAT data in this reader object exists in more than one file. "
+                    "(Call the `.n_files()` method to check.) It is not possible to "
+                    "output data from multiple files to a single local file. "
+                    "To output data, set `is_dir` to `True`, and pass in a directory "
+                    "(not a file path) to `path`."
+                )
+            dir_, basename = os.path.split(path)
+            if not basename and dir_.endswith(os.sep):
+                raise ValueError(
+                    f"You've passed in {dir_} as `path` that looks like a directory "
+                    f"instead of a path to a file, because {dir_} ends with the "
+                    f"directory separator {os.sep}. "
+                    "As an example, a path to a file should look like "
+                    f"foo{os.sep}bar.cha"
+                )
+            elif not basename:
+                raise ValueError(
+                    f"You've passed in {dir_} as `path`, but it doesn't look like "
+                    "a path to a file."
+                )
+            filenames = [basename]
+        else:
+            dir_ = path
+            if filenames is None:
+                filenames = [
+                    f"{str(i + 1).zfill(4)}.cha" for i in range(len(self._files))
+                ]
+            else:
+                filenames = list(filenames)
+            if len(filenames) != len(self._files):
+                raise ValueError(
+                    f"There are {len(self._files)} CHAT files to create, "
+                    f"but you've provided {len(filenames)} filenames."
+                )
+        if dir_:
+            os.makedirs(dir_, exist_ok=True)
+        for filename, lines in zip(filenames, self.to_strs(tabular=tabular)):
+            with open(os.path.join(dir_, filename), "w", encoding=encoding) as f:
+                f.write(lines)
+
     def _parse_chat_str(self, chat_str, file_path) -> _File:
         lines = self._get_lines(chat_str)
         header = self._get_header(lines)
@@ -1246,21 +1594,20 @@ class Reader:
         return index_to_tiers.values()
 
     def _get_header(self, lines: List[str]) -> Dict:
-        headname_to_entry = {"Date": set(), "Participants": {}}
+        headname_to_entry = {}
 
         for line in lines:
 
+            header_re_search = _HEADER_REGEX.search(line)
+            if not header_re_search:
+                continue
+
             if line.startswith("@Begin") or line.startswith("@End"):
                 continue
 
-            if not line.startswith("@"):
-                continue
-
             # find head, e.g., "Languages", "Participants", "ID" etc
-            head, _, line = line.partition("\t")
-            line = line.strip()
-            head = head.lstrip("@")  # remove beginning "@"
-            head = head.rstrip(":")  # remove ending ":", if any
+            head = header_re_search.group(1)
+            line = header_re_search.group(3)
 
             if head == "Participants":
 
@@ -1275,6 +1622,8 @@ class Reader:
                         participant_role,
                     ) = participant_label.partition(" ")
                     # code = participant code, e.g. CHI, MOT
+                    if "Participants" not in headname_to_entry:
+                        headname_to_entry["Participants"] = {}
                     headname_to_entry["Participants"][code] = {"name": participant_name}
 
             elif head == "ID":
@@ -1300,6 +1649,11 @@ class Reader:
                 ]
                 head_to_info = dict(zip(participant_info_heads, participant_info))
 
+                if "Participants" not in headname_to_entry:
+                    headname_to_entry["Participants"] = {}
+                if code not in headname_to_entry["Participants"]:
+                    headname_to_entry["Participants"][code] = {}
+
                 headname_to_entry["Participants"][code].update(head_to_info)
 
             elif head == "Date":
@@ -1307,6 +1661,8 @@ class Reader:
                     date = self._header_line_to_date(line.strip())
                 except (TypeError, ValueError, ParserError):
                     continue
+                if "Date" not in headname_to_entry:
+                    headname_to_entry["Date"] = set()
                 headname_to_entry["Date"].add(date)
 
             elif head.startswith("Birth of"):
@@ -1329,9 +1685,9 @@ class Reader:
                 headname_to_entry["Languages"] = languages
 
             else:
-                headname_to_entry[head] = line
+                headname_to_entry[head] = line or ""
 
-        return headname_to_entry
+        return headname_to_entry if any(headname_to_entry.values()) else {}
 
     @staticmethod
     def _header_line_to_date(line: str) -> datetime.date:
@@ -1340,32 +1696,106 @@ class Reader:
     @staticmethod
     def _get_lines(raw_str: str) -> List[str]:
         lines: List[str] = []
+        raw_str = (raw_str or "").strip()
 
-        previous_line = ""
+        if not raw_str:
+            return lines
 
         for line in raw_str.splitlines():
-            previous_line = previous_line.strip()
-            current_line = line.rstrip()  # don't remove leading \t
-
-            if not current_line:
+            line = line.strip()
+            if not line:
                 continue
 
-            if current_line.startswith("%xpho:") or current_line.startswith("%xmod:"):
-                current_line = current_line.replace("%x", "%", 1)
+            # TODO: Why did I do this?
+            if line.startswith("%xpho:") or line.startswith("%xmod:"):
+                line = line.replace("%x", "%", 1)
 
-            if previous_line and current_line.startswith("\t"):
-                previous_line = f"{previous_line} {current_line.strip()}"
-            elif previous_line:
-                lines.append(previous_line)
-                previous_line = current_line
-            else:  # when it's the very first line
-                previous_line = current_line
+            if line[0] not in _CHAT_LINE_INDICATORS:
+                previous_line = lines.pop()
+                line = f"{previous_line} {line}"
 
-        lines.append(previous_line)  # don't forget the very last line!
+            lines.append(line)
 
         return lines
 
 
+def _get_uuid() -> str:
+    """This function goes hand-in-hand with _is_uuid() below."""
+    return str(uuid.uuid4())
+
+
+def _is_uuid(s: str) -> bool:
+    """This function goes hand-in-hand with _get_uuid() above."""
+    # Implementation from https://stackoverflow.com/a/33245493
+    try:
+        uuid_obj = uuid.UUID(s, version=4)
+    except ValueError:
+        return False
+    return str(uuid_obj) == s
+
+
+def _initialize_cached_data_dir() -> None:
+    if os.path.isdir(_CACHED_DATA_DIR):
+        shutil.rmtree(_CACHED_DATA_DIR)
+    os.makedirs(_CACHED_DATA_DIR)
+    with open(os.path.join(_CACHED_DATA_DIR, "README.txt"), "w", encoding="utf-8") as f:
+        f.write(
+            "The contents of this directory are automatically managed by "
+            "the PyLangAcq library. Please do not edit anything on your own.\n"
+        )
+    with open(_CACHED_DATA_JSON_PATH, "w", encoding="utf-8") as f:
+        f.write("{}")
+
+
+def _write_cached_data_json(records: Dict) -> None:
+    with open(_CACHED_DATA_JSON_PATH, "w", encoding="utf-8") as f:
+        json.dump(records, f, indent=4)
+
+
+def cached_data_info() -> Set[str]:
+    """Return the information of the cached datasets.
+
+    Returns
+    -------
+    Set[str]
+        A set of the URLs for the cached CHILDES / TalkBank datasets.
+    """
+    try:
+        existing_records = json.load(open(_CACHED_DATA_JSON_PATH, encoding="utf-8"))
+    except FileNotFoundError:
+        return set()
+    else:
+        return set(existing_records.keys())
+
+
+def remove_cached_data(url: str = None) -> None:
+    """Remove data cached on disk.
+
+    Parameters
+    ----------
+    url : str, optional
+        If provided, remove only the data specified by this URL. If not provided,
+        all cached data is removed.
+    """
+    try:
+        existing_records = json.load(open(_CACHED_DATA_JSON_PATH, encoding="utf-8"))
+    except FileNotFoundError:
+        _initialize_cached_data_dir()
+        return
+    if url is None:
+        for subdir in [record["subdir"] for record in existing_records.values()]:
+            shutil.rmtree(os.path.join(_CACHED_DATA_DIR, subdir))
+        _initialize_cached_data_dir()
+    else:
+        subdir = existing_records.get(url, {}).get("subdir")
+        if subdir:
+            del existing_records[url]
+            _write_cached_data_json(existing_records)
+            shutil.rmtree(os.path.join(_CACHED_DATA_DIR, subdir))
+        else:
+            raise KeyError(f"url not found among the cached data: {url}")
+
+
 @_params_in_docstring("match", "exclude", "encoding", "cls", class_method=False)
 def read_chat(
     path: str,
@@ -1376,6 +1806,12 @@ def read_chat(
 ) -> "pylangacq.Reader":
     """Create a reader of CHAT data.
 
+    If ``path`` is a remote ZIP file and you expect to call this function
+    with the same path multiple times, consider downloading the data to the local
+    system and then reading it from there to avoid unnecessary re-downloading.
+    Caching a remote ZIP file isn't implemented (yet) as the upstream CHILDES /
+    TalkBank data is updated in minor ways from time to time.
+
     Parameters
     ----------
     path : str
@@ -1463,8 +1899,8 @@ class _HTTPSession(requests.Session):
         return super(_HTTPSession, self).request(*args, **kwargs)
 
 
-def _download_file(url, path):
-    with contextlib.ExitStack() as stack:
-        f = stack.enter_context(open(path, "wb"))
-        r = stack.enter_context(_HTTPSession().get(url, stream=True))
+def _download_file(url, path, session=None):
+    if session is None:
+        session = _HTTPSession()
+    with open(path, "wb") as f, session.get(url, stream=True) as r:
         shutil.copyfileobj(r.raw, f)
